@@ -1,9 +1,10 @@
 #![allow(clippy::type_complexity)]
 
+use std::cell::RefCell;
 pub(crate) mod array;
 pub(crate) mod datetime;
 pub(crate) mod document;
-pub(crate) mod errors;
+pub(crate) mod error;
 pub(crate) mod inline_table;
 pub(crate) mod key;
 pub(crate) mod numbers;
@@ -13,17 +14,21 @@ pub(crate) mod table;
 pub(crate) mod trivia;
 pub(crate) mod value;
 
-pub use errors::TomlError;
+pub(crate) use crate::error::TomlError;
 
-pub(crate) fn parse_document(raw: &str) -> Result<crate::Document, TomlError> {
+pub(crate) fn parse_document<S: AsRef<str>>(raw: S) -> Result<crate::ImDocument<S>, TomlError> {
     use prelude::*;
 
-    let b = new_input(raw);
-    let mut doc = document::document
+    let b = new_input(raw.as_ref());
+    let state = RefCell::new(state::ParseState::new());
+    let state_ref = &state;
+    document::document(state_ref)
         .parse(b)
         .map_err(|e| TomlError::new(e, b))?;
-    doc.span = Some(0..(raw.len()));
-    doc.original = Some(raw.to_owned());
+    let doc = state
+        .into_inner()
+        .into_document(raw)
+        .map_err(|e| TomlError::custom(e.to_string(), None))?;
     Ok(doc)
 }
 
@@ -93,13 +98,16 @@ pub(crate) mod prelude {
         current: usize,
     }
 
+    #[cfg(not(feature = "unbounded"))]
+    const LIMIT: usize = 80;
+
     #[cfg(not(feature = "unbounded"))]
     impl RecursionCheck {
-        pub(crate) fn check_depth(depth: usize) -> Result<(), super::errors::CustomError> {
-            if depth < 128 {
+        pub(crate) fn check_depth(depth: usize) -> Result<(), super::error::CustomError> {
+            if depth < LIMIT {
                 Ok(())
             } else {
-                Err(super::errors::CustomError::RecursionLimitExceeded)
+                Err(super::error::CustomError::RecursionLimitExceeded)
             }
         }
 
@@ -108,13 +116,13 @@ pub(crate) mod prelude {
             input: &mut Input<'_>,
         ) -> Result<Self, winnow::error::ErrMode<ContextError>> {
             self.current += 1;
-            if self.current < 128 {
+            if self.current < LIMIT {
                 Ok(self)
             } else {
                 Err(winnow::error::ErrMode::from_external_error(
                     input,
                     winnow::error::ErrorKind::Eof,
-                    super::errors::CustomError::RecursionLimitExceeded,
+                    super::error::CustomError::RecursionLimitExceeded,
                 ))
             }
         }
@@ -126,7 +134,7 @@ pub(crate) mod prelude {
 
     #[cfg(feature = "unbounded")]
     impl RecursionCheck {
-        pub(crate) fn check_depth(_depth: usize) -> Result<(), super::errors::CustomError> {
+        pub(crate) fn check_depth(_depth: usize) -> Result<(), super::error::CustomError> {
             Ok(())
         }
 
@@ -140,6 +148,8 @@ pub(crate) mod prelude {
 }
 
 #[cfg(test)]
+#[cfg(feature = "parse")]
+#[cfg(feature = "display")]
 mod test {
     use super::*;
 
@@ -182,10 +192,10 @@ hosts = [
     "omega"
 ]
 
-   'some.wierd .stuff'   =  """
+   'some.weird .stuff'   =  """
                          like
                          that
-                      #   """ # this broke my sintax highlighting
+                      #   """ # this broke my syntax highlighting
    " also. like " = '''
 that
 '''
@@ -205,10 +215,7 @@ key = "value"
         ];
         for input in documents {
             dbg!(input);
-            let mut parsed = parse_document(input);
-            if let Ok(parsed) = &mut parsed {
-                parsed.despan();
-            }
+            let parsed = parse_document(input).map(|d| d.into_mut());
             let doc = match parsed {
                 Ok(doc) => doc,
                 Err(err) => {
@@ -233,10 +240,7 @@ authors = []
 "];
         for input in parse_only {
             dbg!(input);
-            let mut parsed = parse_document(input);
-            if let Ok(parsed) = &mut parsed {
-                parsed.despan();
-            }
+            let parsed = parse_document(input).map(|d| d.into_mut());
             match parsed {
                 Ok(_) => (),
                 Err(err) => {
@@ -255,10 +259,7 @@ authors = []
 $"#];
         for input in invalid_inputs {
             dbg!(input);
-            let mut parsed = parse_document(input);
-            if let Ok(parsed) = &mut parsed {
-                parsed.despan();
-            }
+            let parsed = parse_document(input).map(|d| d.into_mut());
             assert!(parsed.is_err(), "Input: {:?}", input);
         }
     }
